#include "single.h"
#include <cassert>
Single::Single(string input, string output): m_input(input), m_output(output)
{
	//figure out relative path.  CL_String::get_path doesn't do a good job with relative paths.. do it manually.

	StringReplace("\\", "/", m_input);
	size_t index = m_input.find_last_of("/");
	if (index != -1)
	{
		//found a path section
		m_inputDir = m_input.substr(0, index+1);
	}

	//same thing for figuring out the output dir
	StringReplace("\\", "/", m_output);
	index = m_output.find_last_of("/");
	if (index != -1)
	{
		//found a path section
		m_outputDir = m_output.substr(0, index+1);
	}


}

Single::~Single()
{

}


void Single::BuildInputFileList()
{
	CL_DirectoryScanner scanner;
	scanner.scan(m_inputDir, CL_String::get_filename(m_input)+"*");

	while (scanner.next())
	{
		std::string file = scanner.get_name();
		if (!scanner.is_directory())
		{
			m_fileArray.push_back(scanner.get_name());
			//CL_Log::log("Sequence", "Adding %1\n", scanner.get_name());
		}
	}

	if (m_fileArray.size() < 1)
	{
		throw CL_Error("Can't find any input files using the dir and search substring of " + m_input);
	}

	CL_Log::log("Sequence", "Found %1 images to process.\n", int(m_fileArray.size()));
}




bool Single::Process()
{
	BuildInputFileList();
	
	//operate on each image separately, we don't handle combined their rects or anything yet
	for (unsigned int i=0; i < m_fileArray.size(); i++)
	{
		Optimize(m_fileArray[i]);
	}
	
	return true; //no erro
}

CL_PixelBuffer GetCellFromImage(int totalFramesX, int totalFramesY, CL_PixelBuffer orig, int frameX, int frameY)
{
	int pixelsPerFrameX = orig.get_width()/totalFramesX;
	int pixelsPerFrameY = orig.get_height()/totalFramesY;
	
	if (pixelsPerFrameX * totalFramesX != orig.get_width())
	{
		throw CL_Error("Image doesn't cleanly divide into requested cell size");
	}
	if (pixelsPerFrameY * totalFramesY != orig.get_height())
	{
		throw CL_Error("Image doesn't cleanly divide into requested cell size");
	}

	assert(frameX < totalFramesX && frameY < totalFramesY);

	CL_PixelBuffer finalBuff(pixelsPerFrameX, pixelsPerFrameY, pixelsPerFrameX*4, CL_PixelFormat::abgr8888);
	
	//copy the cropped position into the new buffer
	CL_Rect destRect = CL_Rect(0,0, pixelsPerFrameX, pixelsPerFrameY);
	

	

	
	CL_Rect srcRect = CL_Rect(frameX*pixelsPerFrameX, frameY*pixelsPerFrameY, frameX*pixelsPerFrameX+pixelsPerFrameX,  frameY*pixelsPerFrameY+pixelsPerFrameY);
	orig.convert(finalBuff.get_data(), finalBuff.get_format(), finalBuff.get_pitch(), destRect, srcRect);
	return finalBuff;
}

void Single::Optimize(string fName)
{
	
	CL_Log::log("", "Optimizing %1 ", m_inputDir+fName);

	CL_PixelBuffer pixBuff = CL_ProviderFactory::load(m_inputDir+fName);
	
	int pixelsPerFrameX = pixBuff.get_width()/App.m_framesX;
	int pixelsPerFrameY = pixBuff.get_height()/App.m_framesY;

	if (pixelsPerFrameX * App.m_framesX != pixBuff.get_width())
	{
		throw CL_Error("Image X doesn't cleanly divide into requested cell size");
	}
	if (pixelsPerFrameY * App.m_framesY != pixBuff.get_height())
	{
		throw CL_Error("Image Y doesn't cleanly divide into requested cell size");
	}
	
	
	//the original cell size
	m_originalRect = CL_Rect(0,0, pixBuff.get_width(), pixBuff.get_height());

	//check each cell for smallest size possible:
	
	bool bFirstImage = true;

	for (int x=0; x < App.m_framesX; x++)
	{
		for (int y=0; y < App.m_framesY; y++)
		{
			CL_PixelBuffer cell = GetCellFromImage(App.m_framesX, App.m_framesY, pixBuff, x,y);
			if (bFirstImage)
			{
				bFirstImage = false;
				m_enclosingRect = GetBoundsFromImage(cell);
				
			} else
			{
				m_enclosingRect = GetBoundsOfCombinedRects(m_enclosingRect, GetBoundsFromImage(cell));
			}
		}
	}

	CL_Rect paddedRect = m_enclosingRect;

	if (App.m_padding > 0) 
	{
		//they want a border..
		paddedRect.right += App.m_padding;
		//m_enclosingRect.left -= App.m_padding;
		//m_enclosingRect.top -= App.m_padding;
		//m_enclosingRect.bottom += App.m_padding;
	}

	//we know what the smallest enclosing rect is.
	CL_Log::log("", "Final bounding Rect [%1, %2, %3, %4] (Size: %5 X %6)\n", paddedRect.left, paddedRect.top, paddedRect.right, m_enclosingRect.bottom,
		paddedRect.get_width(), paddedRect.get_height());



	//write out the new bmp
	
	//image size of the new bmp
	CL_Rect finalRect = CL_Rect(0,0, paddedRect.get_width()*App.m_framesX, paddedRect.get_height()*App.m_framesY);

	CL_PixelBuffer finalBuff(finalRect.get_width(), finalRect.get_height(),finalRect.get_width()*4, CL_PixelFormat::abgr8888);
	ClearPixelBuffer(&finalBuff, CL_Color(0,0,0,0));
	
	//copy over the rects 1 by 1 into the correct spots
	for (int x=0; x < App.m_framesX; x++)
	{
		for (int y=0; y < App.m_framesY; y++)
		{
			CL_Rect destRect = CL_Rect(x*paddedRect.get_width(), y*paddedRect.get_height(), x*paddedRect.get_width()+m_enclosingRect.get_width(),
			y*paddedRect.get_height()+m_enclosingRect.get_height());
		CL_PixelBuffer cell = GetCellFromImage(App.m_framesX, App.m_framesY, pixBuff, x,y);
		cell.convert(finalBuff.get_data(), finalBuff.get_format(), finalBuff.get_pitch(), destRect, m_enclosingRect);
		}
	}

	string fileName = GetValidOutputType(m_outputDir+fName);
	CL_ProviderFactory::save(finalBuff, fileName);

}